
import re
from typing import List, Optional, Union

import unicodedata


class StringUtils:
    """
    字符串处理工具类
    """
    @staticmethod
    def pick_head(s: str, keyword: str) -> Optional[str]:
        return s.split(keyword)[0] if s else None

    @staticmethod
    def pick_tail(s: str, keyword: str) -> Optional[str]:
        if s is None or keyword not in s:
            return None

        return s.split(keyword, 1)[1]

    @staticmethod
    def pick_numbers(text: str) -> List[Union[int, float]]:
        numbers = []

        matches = re.findall(r"\b\d+\.\d+|\b\d+", text)
        for match in matches:
            # 判断是整数还是浮点数并转换
            if '.' in match:
                numbers.append(float(match))
            else:
                numbers.append(int(match))
        return numbers

    @staticmethod
    def pick_number(text: str) -> Optional[Union[int, float]]:
        numbers = StringUtils.pick_numbers(text)
        return numbers[0] if len(numbers) > 0 else None

    @staticmethod
    def split_by_keyword(input_string: str, keyword: str = '{}'):
        result = None, None, None
        if input_string:
            first, end = keyword[0], keyword[1]
            if first not in input_string or end not in input_string:
                return input_string, None, None
            else:
                pattern = rf'^(.*?)(\{first}(.*?)\{end})(.*)$'
                match = re.match(pattern, input_string)
                if match:
                    result1 = match.group(1) if match.group(1) else None # 第一个捕获组
                    result2 = match.group(3) if match.group(3) else None  # 第三个捕获组
                    result3 = match.group(4) if match.group(4) else None  # 第四个捕获组
                    result = result1, result2, result3
                else:
                    raise ValueError("输入字符串格式不正确")
        return result

    @staticmethod
    def split_name_id(text: str) -> {str, str}:
        """
        将 名称(标识)字符串分为{名称, 标识}
        :param text: 名称(标识)字符串
        :return: {名称, 标识}
        """
        match = re.match(r"(.+)[(（](.+)[)）]", text)
        if match:
            _name = match.group(1)
            _id = match.group(2)
            return _name, _id
        else:
            return text, None

    @staticmethod
    def parse_function(function_name: str) -> tuple:
        """
        解析字符串，将其分解为方法名和参数
        :param function_name: 字符串，格式如：test(arg1, arg2)
        :return: 方法名和参数列表
        """
        pattern = r'(\w+)\((.*)\)'
        match = re.match(pattern, function_name)

        if not match:
            raise ValueError("字符串格式不正确")

        method_name = match.group(1)
        args = match.group(2).split(',') if match.group(2) else []

        # 去除参数两端的空格
        args = [arg.strip() for arg in args]

        return method_name, args

    @staticmethod
    def to_chinese_number(num: int) -> str:
        units = ["", "十", "百", "千", "万", "十", "百", "千", "亿"]
        digits = ["零", "一", "二", "三", "四", "五", "六", "七", "八", "九"]

        if num == 0:
            return "零"

        result = ""
        unit_position = 0
        while num > 0:
            digit = num % 10
            if digit != 0:
                result = f"{digits[digit]}{units[unit_position]}{result}"
            elif result and result[0] != "零":
                result = "零" + result
            num //= 10
            unit_position += 1

        # 处理 "一十" 的情况
        if result.startswith("一十"):
            result = result[1:]

        return result

    @staticmethod
    def parse_strings_to_map(strs: List[str], delimiter: str = ' ') -> dict[str, str]:
        """
        将字符串数组解析为字典。

        参数：
        :param strs: 字符串数组
        :param delimiter: 分隔符，默认为空格
        :return: 字典
        """
        data_map = {}
        for _str in strs:
            if delimiter in _str:
                key, value = _str.split(delimiter, maxsplit=1)
                data_map[key] = value
            else:
                raise ValueError(f"字符串 '{_str}' 中没有分隔符 '{delimiter}'，无法解析为键值对")
        return data_map

    @staticmethod
    def get_print_width(s: str, chines_width: float = 1.67) -> int:
        width = 0
        for char in s:
            if unicodedata.east_asian_width(char) in ('F', 'W'):
                width += chines_width
            else:
                width += 1
        return int(width)

    @staticmethod
    def remove_between_keywords(text: str, keyword_begin: str, keyword_end: str) -> str:
        # 使用正则表达式去除keyword_begin和keyword_end之间的内容，包括这两个关键词
        pattern = re.escape(keyword_begin) + '.*?' + re.escape(keyword_end)
        return re.sub(pattern, '', text, flags=re.DOTALL)



    @staticmethod
    def underline2hump(string: str) -> str:
        """
        将用"_"拼接的字符串转换为驼峰格式，首字母小写。

        :param string: String 参数
        """
        # 首字母小写
        string = StringUtils.lower_first_letter(string)
        return re.sub(r'_([a-z])', lambda x: x.group(1).upper(), string)

    @staticmethod
    def hump2underline(s: str) -> str:
        """
        将驼峰格式参数变换为"_"连接的格式。
        """
        s = StringUtils.lower_first_letter(s)
        hump_pattern = re.compile(r'([A-Z])')
        return hump_pattern.sub(r'_\1', s).lower()

    @staticmethod
    def lower_first_letter(string: str) -> str:
        """
        将字符串的首字母转换为小写

        :param string: 输入字符串
        :return: 首字母小写的字符串
        """
        return string[0].lower() + string[1:] if string else ""

    @staticmethod
    def upper_first_letter(string: str) -> str:
        """
        将字符串的首字母转换为小写

        :param string: 输入字符串
        :return: 首字母小写的字符串
        """
        return string[0].upper() + string[1:] if string else ""

    @staticmethod
    def last_word(s: str) -> str:
        """
        获取字段串末尾的关键词，空格分隔

        :param s: 原字符串
        :return: 末尾的关键词
        """
        tokens = s.split()
        return tokens[-1] if tokens else ""


from typing import List, Union

import numpy as np

from mag_tools.utils.data.value_utils import ValueUtils


class ArrayUtils:
    """
    数组工具类，在数值计算中通常采用np.ndarray
    """
    @staticmethod
    def text_to_array_1d(text: str, data_type: type) -> np.ndarray:
        """
        从一行文本中获取数值数组[]
        :param text: 文本，由 "数值" 或 "数值个数*数值"构成
        :param data_type: 数据类型
        :return: 数值数组
        """
        data = []

        for part in text.split():
            if '*' in part:
                count, value = part.split('*')
                data.extend([ValueUtils.to_value(value, data_type)] * int(count))
            else:
                data.append(ValueUtils.to_value(part, data_type))

        return np.array(data, dtype=data_type)

    @staticmethod
    def lines_to_array_1d(lines: List[str], data_type: type) -> np.ndarray:
        """
        从多行文本中获取数值数组[]
        :param lines: 文本段，每行由 "数值" 或 "数值个数*数值"构成
        :param data_type: 数据类型
        :return: 数值数组
        """
        array_1d = []
        for line in lines:
            array_1d.extend(ArrayUtils.text_to_array_1d(line, data_type).tolist())
        return np.array(array_1d, dtype=data_type)

    @staticmethod
    def lines_to_array_2d(lines: List[str], nx: int, ny: int, data_type: type) -> np.ndarray:
        """
        从文本块获取数值列表
        :param lines: 文本块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        :param nx: 行数
        :param ny: 列数
        :param data_type: 数据类型
        :return: Values
        """
        array_1d = ArrayUtils.lines_to_array_1d(lines, data_type)

        if len(array_1d) != nx * ny:
            raise ValueError(
                f"Data length mismatch: Expected {nx}*{ny}, but got {len(array_1d)}."
            )
        return array_1d.reshape((nx, ny))

    @staticmethod
    def lines_to_array_3d(block_lines: List[str], nx: int, ny: int, nz: int, data_type: type) -> np.ndarray:
        """
        从文本块获取数值列表
        :param block_lines: 文本块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        :param nx: 行数
        :param ny: 列数
        :param nz: 层数
        :param data_type: 数据类型
        :return: Values
        """
        array_1d = ArrayUtils.lines_to_array_1d(block_lines, data_type)
        return array_1d.reshape((nx, ny, nz))

    @staticmethod
    def assign_array_2d(array2: np.ndarray, i1: int, i2: int, j1: int, j2: int, value: Union[int, float]):
        """
        给数组指定区域的元素赋值
        :param array2: 二维数组
        :param i1: 起始行数
        :param i2: 结束行数
        :param j1: 起始列数
        :param j2: 结束列数
        :param value: 数值
        """
        rows, cols = array2.shape

        if i1 < 0 or j1 < 0 or i2 >= rows or j2 >= cols or i1 > i2 or j1 > j2:
            raise IndexError("Index out of range for the provided 2D array dimensions.")

        array2[i1:i2 + 1, j1:j2 + 1] = value
        return array2

    @staticmethod
    def multiply_array_2d(array2: np.ndarray, i1: int, i2: int, j1: int, j2: int, factor: Union[int, float]):
        """
        给数组指定区域的元素乘以系数
        :param array2: 二维数组
        :param i1: 起始行数
        :param i2: 结束行数
        :param j1: 起始列数
        :param j2: 结束列数
        :param factor: 系数
        """
        rows, cols = array2.shape
        if i1 < 0 or j1 < 0 or i2 >= rows or j2 >= cols or i1 > i2 or j1 > j2:
            raise IndexError("Index out of range for the provided 2D array dimensions.")

        array2[i1:i2 + 1, j1:j2 + 1] *= factor
        return array2

    @staticmethod
    def add_array_2d(array2: np.ndarray, i1: int, i2: int, j1: int, j2: int, value: Union[int, float]):
        """
        给数组指定区域的元素增加数值
        :param array2: 二维数组
        :param i1: 起始行数
        :param i2: 结束行数
        :param j1: 起始列数
        :param j2: 结束列数
        :param value: 数值
        """
        rows, cols = array2.shape
        if i1 < 0 or j1 < 0 or i2 >= rows or j2 >= cols or i1 > i2 or j1 > j2:
            raise IndexError("Index out of range for the provided 2D array dimensions.")

        array2[i1:i2 + 1, j1:j2 + 1] += value
        return array2

    @staticmethod
    def assign_array_3d(array3: np.ndarray, i1: int, i2: int, j1: int, j2: int, k1: int, k2: int, value: Union[float, int]):
        """
        给数组指定区域的元素赋值
        :param array3: 三维数组
        :param i1: 起始层数
        :param i2: 结束层数
        :param j1: 起始行数
        :param j2: 结束行数
        :param k1: 起始列数
        :param k2: 结束列数
        :param value: 数值
        """
        layers, rows, cols = array3.shape
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= layers or j2 >= rows or k2 >= cols or i1 > i2 or j1 > j2 or k1 > k2:
            raise IndexError("Index out of range for the provided 3D array dimensions.")

        array3[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1] = value
        return array3

    @staticmethod
    def multiply_array_3d(array3: np.ndarray, i1: int, i2: int, j1: int, j2: int, k1: int, k2: int, factor: Union[int, float]):
        """
        给数组指定区域的元素乘以系数
        :param array3: 三维数组
        :param i1: 起始层数
        :param i2: 结束层数
        :param j1: 起始行数
        :param j2: 结束行数
        :param k1: 起始列数
        :param k2: 结束列数
        :param factor: 系数
        """
        layers, rows, cols = array3.shape
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= layers or j2 >= rows or k2 >= cols or i1 > i2 or j1 > j2 or k1 > k2:
            raise IndexError("Index out of range for the provided 3D array dimensions.")

        array3[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1] *= factor
        return array3

    @staticmethod
    def add_array_3d(array3: np.ndarray, i1: int, i2: int, j1: int, j2: int, k1: int, k2: int, value: Union[float,int]):
        """
        给数组指定区域的元素增加数值
        :param array3: 三维数组
        :param i1: 起始层数
        :param i2: 结束层数
        :param j1: 起始行数
        :param j2: 结束行数
        :param k1: 起始列数
        :param k2: 结束列数
        :param value: 数值
        """
        layers, rows, cols = array3.shape
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= layers or j2 >= rows or k2 >= cols or i1 > i2 or j1 > j2 or k1 > k2:
            raise IndexError("Index out of range for the provided 3D array dimensions.")

        array3[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1] += value
        return array3

    @staticmethod
    def copy_array_3d(source_array: np.ndarray, target_array: np.ndarray, i1: int, i2: int, j1: int, j2: int, k1: int, k2: int):
        """
        复制源数组指定区域到目标数组
        :param source_array: 源数组
        :param target_array: 目标数组
        :param i1: 起始层数
        :param i2: 结束层数
        :param j1: 起始行数
        :param j2: 结束行数
        :param k1: 起始列数
        :param k2: 结束列数
        """

        src_layers, src_rows, src_cols = source_array.shape
        tgt_layers, tgt_rows, tgt_cols = target_array.shape

        # 检查索引是否超出源数组的范围
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= src_layers or j2 >= src_rows or k2 >= src_cols:
            raise IndexError(
                f"Source array index out of range: i1={i1}, i2={i2}, j1={j1}, j2={j2}, k1={k1}, k2={k2}, source_shape={source_array.shape}"
            )

        # 检查索引是否超出目标数组的范围
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= tgt_layers or j2 >= tgt_rows or k2 >= tgt_cols:
            raise IndexError(
                f"Target array index out of range: i1={i1}, i2={i2}, j1={j1}, j2={j2}, k1={k1}, k2={k2}, target_shape={target_array.shape}"
            )

        # 检查索引逻辑是否合理 (例如 i1 > i2)
        if i1 > i2 or j1 > j2 or k1 > k2:
            raise ValueError(
                f"Invalid indices: i1={i1}, i2={i2}, j1={j1}, j2={j2}, k1={k1}, k2={k2}. Start index must not be greater than end index."
            )

        target_array[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1] = source_array[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1]
        return target_array



from dataclasses import MISSING, dataclass, field, fields
from typing import Any, Dict

from mag_tools.format.text_formatter import TextFormatter
from mag_tools.model.justify_type import JustifyType


@dataclass
class BaseData:
    """
    Bean的基类
    """
    _formatter = TextFormatter(
        number_per_line=None,
        justify_type=JustifyType.LEFT,
        at_header='',
        decimal_places=6,
        decimal_places_of_zero=2,
        pad_length=None,
        pad_char=' ',
        scientific=False
    )

    def __repr__(self):
        field_dict = {k: v for k, v in self.__dict__.items() if not k.startswith('_')}
        field_str = ', '.join(f"{k}={v}" for k, v in field_dict.items())
        return f"{self.__class__.__name__}({field_str})"

    def __str__(self):
        excluded_attributes = {}
        attributes = [
            f"{attr}={repr(getattr(self, attr))}"
            for attr in vars(self)
            if getattr(self, attr) is not None and attr not in excluded_attributes
        ]
        return f"{self.__class__.__name__}({', '.join(attributes)})"

    @classmethod
    def get_default_value(cls, field_name: str) -> Any:
        """
        获取字段的缺省值
        """
        for f in fields(cls):   # type: ignore
            if f.name == field_name:
                return f.default if f.default is not MISSING else None
        raise ValueError(f"Field '{field_name}' not found in {cls.__name__}")

    @property
    def to_map(self) -> Dict[str, Any]:
        return {
            k: repr(v) for k, v in self.__dict__.items()
            if not (k.startswith('__') and k.endswith('__')) and k not in {'_text_format', '_data_formats'}
        }

@dataclass
class TestData(BaseData):
    name: str = field(default=None, metadata={"description": "UUID"})
    age: int = field(default=12, metadata={"description": "UUID"})
    height: float = field(default=1.56, metadata={"description": "UUID"})


if __name__ == '__main__':
    data = TestData(name='xlcao', age=12, height=1)
    age_ = TestData.get_default_value('height')
    print(age_)


from datetime import datetime, timedelta

from dateutil import parser


class DateUtils:
    """
    日期时间的工具类
    """
    @staticmethod
    def float_to_datetime(timestamp: float) -> datetime:
        """
        将浮点数转换为日期时间对象

        :param timestamp: 浮点数，表示自 Unix 纪元以来的秒数
        :return: 日期时间对象
        """
        return datetime(1970, 1, 1) + timedelta(seconds=timestamp)

    @staticmethod
    def to_datetime(date_time_str):
        date_time_formats = [
            "%Y-%m-%d %H:%M:%S",
            "%Y/%m/%d %H:%M:%S",
            "%Y.%m.%d %H:%M:%S",
            "%Y年%m月%d日 %H时%M分%S秒",
            "%Y-%m-%d",
            "%Y/%m/%d",
            "%Y.%m.%d",
            "%H:%M:%S",
            "%H时%M分%S秒",
            "%Y-%m-%d %H:%M",
            "%Y-%m-%d %H:%M:%S.%f",
            "%Y%m%d%H%M%S",
            "%Y%m%d%H%M%S%f",
            "%Y%m%d",
            "%a, %d %b %Y %H:%M:%S %Z",
            "%a %b %d %H:%M:%S %Z %Y",
            "%Y-%m-%dT%H:%M:%S%z",
            "%Y-%m-%dT%H:%M:%S.%fZ",
            "%Y-%m-%dT%H:%M:%S%z",
            "%Y-%m-%dT%H:%M:%S.%f%z"
        ]

        for fmt in date_time_formats:
            try:
                return datetime.strptime(date_time_str, fmt)
            except ValueError:
                continue

        try:
            return parser.parse(date_time_str)
        except ValueError:
            return None



from typing import Any, Optional, Union

import numpy as np


class ValueUtils:
    @staticmethod
    def to_value(text: str, data_type: Optional[type] = None):
        """
        将文本转换为数值
        :param text: 文本
        :param data_type: 数据类型
        """
        if text is None or not isinstance(text, str):
            return None

        text = text.strip()
        if data_type == int:
            try:
                return int(text)
            except ValueError:
                return None
        elif data_type == float:
            try:
                if text.endswith('%'):
                    return float(text[:-1]) / 100

                return float(text)
            except ValueError:
                return None
        elif data_type == bool:
            text = text.lower()
            return text in ['true', 'yes', 't', 'y', '1']
        elif data_type == list:
            return eval(text)
        elif data_type == dict:
            return eval(text)
        else:
            return text

    @staticmethod
    def to_string(value: Union[int, float, bool, str], decimal_places: int=6, decimal_places_of_zero: int=1,
                  scientific: bool = False) -> str:
        if isinstance(value, str) or isinstance(value, bool):
            return str(value)
        else:
            if abs(value) >= 1e8 or (value != 0 and abs(value) < 1e-3):
                scientific = True
            elif isinstance(value, int):
                scientific = False

            if scientific:
                return ValueUtils.number_to_scientific(value, decimal_places)

            if isinstance(value, float):
                if value.is_integer():
                    return f"{value:.{decimal_places_of_zero}f}"
                else:
                    formatted_value = f"{value:.{decimal_places}f}"
                    # 确保小数位末尾的零的个数符合要求
                    if '.' in formatted_value:
                        integer_part, decimal_part = formatted_value.split('.')
                        decimal_part = decimal_part.rstrip('0')
                        if len(decimal_part) < decimal_places_of_zero:
                            decimal_part += '0' * (decimal_places_of_zero - len(decimal_part))
                        formatted_value = f"{integer_part}.{decimal_part}"
                    return formatted_value
            elif isinstance(value, bool):
                return str(value)
            elif isinstance(value, int):
                return f"{round(value)}"

    @staticmethod
    def to_native(value: Any) -> Any:
        """
        将值转换为原生类型，如果可能
        :param value: 原始值
        :return: 转换后的原生类型值
        """
        if isinstance(value, (np.integer, int)):
            return int(value)  # 转换为 Python 的 int 类型
        elif isinstance(value, (np.floating, float)):
            return float(value)  # 转换为 Python 的 float 类型
        elif isinstance(value, (np.bool_, bool)):
            return bool(value)  # 转换为 Python 的 bool 类型
        else:
            return str(value)  # 保留为字符串

    @staticmethod
    def number_to_scientific(value: Union[int,float], decimal_places: int = 6) -> str:
        """
        将 float 数字转换为科学计数法表示的字符串。

        参数：
        :param value: float 数字
        :param decimal_places: 小数位数，默认为 6
        :return: 科学计数法表示的字符串
        """
        exponent = int(f"{value:e}".split('e')[1])
        coefficient = f"{value / (10 ** exponent):.{decimal_places}f}".rstrip('0').rstrip('.')
        if '.' in coefficient:
            integer_part, decimal_part = coefficient.split('.')
            if len(decimal_part) < decimal_places:
                decimal_part += '0' * (decimal_places - len(decimal_part))
            coefficient = f"{integer_part}.{decimal_part}"
        else:
            coefficient += '.' + '0' * decimal_places
        return f"{coefficient}e{exponent}"

    @staticmethod
    def get_type(value: Any) -> Optional[type]:
        """
        判断输入值的数据类型

        参数:
        value: 要判断的数据

        返回:
        str: 数据类型
        """
        if isinstance(value, int):
            return int
        elif isinstance(value, float):
            return float
        elif isinstance(value, complex):
            return complex
        elif isinstance(value, bool):
            return bool
        elif isinstance(value, str):
            return str
        else:
            return None


import unittest

from mag_tools.format.text_formatter import TextFormatter

from mag_tools.model.justify_type import JustifyType


class TestTextFormat(unittest.TestCase):
    def setUp(self):
        self.format = TextFormatter(
            number_per_line=3,
            justify_type=JustifyType.LEFT,
            at_header='   ',
            decimal_places=3,
            decimal_places_of_zero=3,
            pad_length=None,
            pad_char=' ',
            scientific=False
        )

    def test_to_text(self):
        values = [12.345, 12, 'home', -1232.0000, 123456789010, 1.12e-9, 1.23e5]
        print(f"\n{'\n'.join(self.format.to_lines(values))}")

    def test_to_block(self):
        values = [[12.345, 12, 'home', -1232.0000, 123456789010, 1.12e-9, 1.23e5],
                  [12345.67, 16, 'World', 1232.120, 12345, 1.22e-6, 1.20e3],
                  [122321.345000, 19, 'Earth', 32.02, 6789010, 1.12e3, 2.23e2]]
        print(f"\n{'\n'.join(self.format.to_blocks(values))}")

    def test_array_1d_to_lines(self):
        text_format = TextFormatter(number_per_line=3, justify_type=JustifyType.LEFT)

        # 测试整数数组
        array_1d = [1, 1, 2, 3, 3, 3]
        expected_output = ['2*1 2   3*3']
        result = text_format.array_1d_to_lines(array_1d)
        self.assertEqual(result, expected_output)

        # 测试浮点数数组
        array_1d = [1.1, 1.1, 2.2, 3.3, 3.3, 3.3]
        expected_output = ['2*1.1 2.2   3*3.3']
        result = text_format.array_1d_to_lines(array_1d)
        self.assertEqual(result, expected_output)

        # 测试布尔值数组
        array_1d = [True, True, False, True, True, True]
        expected_output = ['2*True False  3*True']
        result = text_format.array_1d_to_lines(array_1d)
        self.assertEqual(result, expected_output)

        # 测试字符串数组
        array_1d = ['a', 'a', 'b', 'c', 'c', 'c']
        expected_output = ['2*a b   3*c']
        result = text_format.array_1d_to_lines(array_1d)
        self.assertEqual(result, expected_output)

    def test_array_2d_to_lines(self):
        text_format = TextFormatter(number_per_line=3, justify_type=JustifyType.LEFT)

        # 测试整数数组
        array_2d = [
            [1, 1, 2],
            [3, 3, 3]
        ]
        expected_output = ['2*1 2   3*3']
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        # 测试浮点数数组
        array_2d = [
            [1.1, 1.1, 2.2],
            [3.3, 3.3, 3.3]
        ]
        expected_output = ['2*1.1 2.2   3*3.3']
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        # 测试布尔值数组
        array_2d = [
            [True, True, False],
            [True, True, True]
        ]
        expected_output = ['2*True False  3*True']
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

        # 测试字符串数组
        array_2d = [
            ['a', 'a', 'b'],
            ['c', 'c', 'c']
        ]
        expected_output = ['2*a b   3*c']
        result = text_format.array_2d_to_lines(array_2d)
        self.assertEqual(result, expected_output)

    def test_array_3d_to_lines(self):
        text_format = TextFormatter(number_per_line=3, justify_type=JustifyType.LEFT)

        # 测试整数数组
        array_3d = [
            [
                [1, 1, 2],
                [3, 3, 3]
            ],
            [
                [4, 4, 5],
                [6, 6, 6]
            ]
        ]
        expected_output = ['2*1 2   3*3', '2*4 5   3*6']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        # 测试浮点数数组
        array_3d = [
            [
                [1.1, 1.1, 2.2],
                [3.3, 3.3, 3.3]
            ],
            [
                [4.4, 4.4, 5.5],
                [6.6, 6.6, 6.6]
            ]
        ]
        expected_output = ['2*1.1 2.2   3*3.3', '2*4.4 5.5   3*6.6']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        # 测试布尔值数组
        array_3d = [
            [
                [True, True, False],
                [True, True, True]
            ],
            [
                [False, False, True],
                [False, False, False]
            ]
        ]
        expected_output = ['2*True  False   3*True ', '2*False True    3*False']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)

        # 测试字符串数组
        array_3d = [
            [
                ['a', 'a', 'b'],
                ['c', 'c', 'c']
            ],
            [
                ['d', 'd', 'e'],
                ['f', 'f', 'f']
            ]
        ]
        expected_output = ['2*a b   3*c', '2*d e   3*f']
        result = text_format.array_3d_to_lines(array_3d)
        self.assertEqual(expected_output, result)


import logging
import logging.config
import os
import traceback

from mag_tools.utils.data.string_utils import StringUtils

from mag_tools.config.sys_config import SysConfig
from mag_tools.exception.app_exception import AppException
from mag_tools.model.log_type import LogType


class Logger:
    __instance = None

    def __init__(self):
        """
        初始化 Logger 实例，配置日志记录器和日志文件路径。
        """
        log_dir = SysConfig.get('logDir')
        if log_dir is None:
            log_dir = os.path.join(SysConfig.root_dir(), 'data', 'log')


        os.makedirs(log_dir, exist_ok=True)

        logging.config.fileConfig(SysConfig.logging_conf(),
                                  encoding='utf-8',
                                  defaults={'logdir': str(log_dir)})

        self.root_logger = logging.getLogger()
        self.frame_logger = logging.getLogger('frame')
        self.dao_logger = logging.getLogger('dao')
        self.service_logger = logging.getLogger('service')
        self.performance_logger = logging.getLogger('performance')

    @classmethod
    def __get_instance(cls):
        if not cls.__instance:
            cls.__instance = cls()
        return cls.__instance

    @classmethod
    def debug(cls, *args):
        """
        记录调试级别的日志信息。

        :param args: 日志信息参数，可以是单个消息或 (logger_type, message) 元组
        """
        if len(args) == 1:
            message = args[0]
            cls.__get_instance().__debug(LogType.FRAME, message)
        elif len(args) == 2:
            log_type, message = args
            cls.__get_instance().__debug(log_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @classmethod
    def info(cls, *args):
        """
        记录信息级别的日志信息。

        :param args: 日志信息参数，可以是单个消息、(logger_type, message) 或 (logger_type, message, is_highlight) 元组
        """
        if len(args) == 1:
            message = args[0]
            cls().__info(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            cls().__info(logger_type, message)
        elif len(args) == 3:
            logger_type, message, is_highlight = args
            if is_highlight:
                cls().__info(logger_type, '*' * (StringUtils.get_print_width(message) + 8))
                cls.__get_instance().__info(logger_type, f'*** {message} ***')
                cls.__get_instance().__info(logger_type, '*' * (StringUtils.get_print_width(message) + 8))
            else:
                cls().__info(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @classmethod
    def warning(cls, *args):
        """
        记录警告级别的日志信息。

        :param args: 日志信息参数，可以是单个消息或 (logger_type, message) 元组
        """
        if len(args) == 1:
            message = args[0]
            cls.__get_instance().__warning(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            cls.__get_instance().__warning(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @classmethod
    def error(cls, *args):
        """
        记录错误级别的日志信息。

        :param args: 日志信息参数，可以是单个消息或 (logger_type, message) 元组
        """
        if len(args) == 1:
            message = str(args[0]) if isinstance(args[0], Exception) else args[0]
            cls.__get_instance().__error(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            message = str(message) if isinstance(message, Exception) else message
            cls.__get_instance().__error(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @classmethod
    def throw(cls, *args):
        Logger.error(args)

        if len(args) == 1:
            message = str(args[0]) if isinstance(args[0], Exception) else args[0]
        elif len(args) == 2:
            logger_type, message = args
            message = str(message) if isinstance(message, Exception) else message
        else:
            raise ValueError("Invalid number of arguments")

        raise AppException(message)

    def __debug(self, logger_type: LogType, message: str):
        """
        内部方法，记录调试级别的日志信息。

        :param logger_type: 日志类型
        :param message: 日志消息
        """
        self.root_logger.debug(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.debug(message)
        elif logger_type == LogType.DAO:
            self.dao_logger.debug(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.debug(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.debug(message)

    def __info(self, logger_type: LogType, message: str):
        """
        内部方法，记录信息级别的日志信息。

        :param logger_type: 日志类型
        :param message: 日志消息
        """
        self.root_logger.info(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.info(message)
        elif logger_type == LogType.DAO:
            self.dao_logger.info(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.info(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.info(message)

    def __warning(self, logger_type: LogType, message: str):
        """
        内部方法，记录警告级别的日志信息。

        :param logger_type: 日志类型
        :param message: 日志消息
        """
        self.root_logger.warning(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.warning(message)
        elif logger_type == LogType.DAO:
            self.dao_logger.warning(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.warning(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.warning(message)

    def __error(self, logger_type: LogType, message: str):
        """
        内部方法，记录错误级别的日志信息。

        :param logger_type: 日志类型
        :param message: 日志消息
        """
        error_message = f"{message}\n{traceback.format_exc()}"
        self.root_logger.error(error_message)

        if logger_type == LogType.FRAME:
            self.frame_logger.error(error_message)
        elif logger_type == LogType.DAO:
            self.dao_logger.error(error_message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.error(error_message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.error(error_message)